from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, SetAddonStatus
from RSTAB.enums import GlobalParameterUnitGroup, GlobalParameterDefinitionType, AddOn

class GlobalParameter():

    @staticmethod
    def AddParameter(
                     no: int = 1,
                     name: str = '',
                     symbol: str = '',
                     unit_group = GlobalParameterUnitGroup.LENGTH,
                     definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_VALUE,
                     definition_parameter: list = None,
                     comment: str = '',
                     params: dict = None,
                     model = Model):
        '''
        Args:
            no (int): Global Parameter Tag
            name (str): Parameter Name
            symbol (str): Symbol
            unit_group (enum): Global Parameter Unit Group Enumeration
            definition_type (enum): Global Parameter Definition Type Enumeration
            definition_parameter (list): Definition Parameter List
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_FORMULA:
                    definition_parameter = [formula]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_OPTIMIZATION:
                    definition_parameter = [min, max, increment, steps]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_OPTIMIZATION_ASCENDING:
                    definition_parameter = [min, max, increment, steps]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_OPTIMIZATION_DESCENDING:
                    definition_parameter = [value, min, max, steps]
                for definition_type = GlobalParameterDefinitionType.DEFINITION_TYPE_VALUE:
                    definition_parameter = [value]
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Global Parameter
        clientObject = model.clientModel.factory.create('ns0:global_parameter')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Global Parameter No.
        clientObject.no = no

        # Global Parameter Name
        clientObject.name = name

        # Symbol (HTML)
        clientObject.symbol = symbol

        # Unit Group
        clientObject.unit_group = unit_group.name

        # Definition Type
        clientObject.definition_type = definition_type.name

        if definition_type.name == 'DEFINITION_TYPE_FORMULA':
            if len(definition_parameter) != 1:
                raise ValueError('WARNING: The definition parameter needs to be of length 1. Kindly check list inputs for completeness and correctness.')
            clientObject.formula = definition_parameter[0]

        elif definition_type.name == 'DEFINITION_TYPE_OPTIMIZATION' or definition_type.name == 'DEFINITION_TYPE_OPTIMIZATION_ASCENDING' or definition_type.name == 'DEFINITION_TYPE_OPTIMIZATION_DESCENDING':
            SetAddonStatus(model.clientModel, AddOn.cost_estimation_active)
            if len(definition_parameter) != 4:
                raise ValueError('WARNING: The definition parameter needs to be of length 4. Kindly check list inputs for completeness and correctness.')
            clientObject.value = definition_parameter[0]
            clientObject.min = definition_parameter[1]
            clientObject.max = definition_parameter[2]
            clientObject.steps = definition_parameter[3]

        elif definition_type.name == 'DEFINITION_TYPE_VALUE':
            if len(definition_parameter) != 1:
                raise ValueError('WARNING: The definition parameter needs to be of length 1. Kindly check list inputs for completeness and correctness.')
            clientObject.value = definition_parameter[0]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Global Parameter to client model
        model.clientModel.service.set_global_parameter(clientObject)



from RSTAB.initModel import Model
from RSTAB.enums import *

class OptimizationSettings():
    def __init__(self,
        general_optimization_active = True,
        general_keep_best_number_model_mutations = 10,
        general_optimize_on = OptimizeOnType.E_OPTIMIZE_ON_TYPE_MIN_WHOLE_WEIGHT,
        general_optimizer = Optimizer.E_OPTIMIZER_TYPE_ALL_MUTATIONS,
        general_number_random_mutations = 0.2,
        model = Model):
        """
        The object is automaticaly created therefore we can assume,
        that it will not be created but only updated/changed.

        Args:
            general_optimization_active (bool): Set Optimization Active
            general_keep_best_number_model_mutations (int): Keep best number of model mutations
            general_optimize_on (enum): Optimize on
            general_optimizer (enum): Optimizer
            general_number_random_mutations (float): Number of random mutations
        """

        opt_settings = model.clientModel.service.get_optimization_settings()
        opt_settings.general_optimization_active = general_optimization_active
        opt_settings.general_keep_best_number_model_mutations = general_keep_best_number_model_mutations
        opt_settings.general_optimize_on = general_optimize_on.name
        opt_settings.general_optimizer = general_optimizer.name
        opt_settings.general_number_random_mutations = general_number_random_mutations

        # Set Optimization settings to client model
        model.clientModel.service.set_optimization_settings(opt_settings)

    @staticmethod
    def get(model = Model):
        return model.clientModel.service.get_optimization_settings()

    @staticmethod
    def set(opt_settings, model = Model):
        model.clientModel.service.set_optimization_settings(opt_settings)


from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import PositionOnSection

class TimberMemberShearPanel():
    def __init__(self,
                no: int = 1,
                name: str = '',
                members: str = "",
                member_sets: str = "",
                position_on_section = PositionOnSection.POSITION_ON_UPPER_FLANGE,
                stiffness: float = 1000,
                position_on_section_value: float = 0.005,
                comment: str = '',
                params: dict = None):
        """
        Args:
            no (int): Timber Member Shear Panel Tag
            name (str): User Defined Member Shear Panel Name
            definition_type (enum): Timber Member Shear Panel Definition Type Enumeration
            members (str): Assigned Members
            member_sets (str): Assigned Member Sets
            position_on_section (enum): Position On Section
            stiffness (float): Shear Panel Stiffness
            position_on_section_value (float): Position On Section Value if POsition On Section equals POSITION_DEFINE
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
        """

         # Client Model | Types For Timber Design Member Shear Panel
        clientObject = Model.clientModel.factory.create('ns0:timber_member_shear_panel')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Shear Panel No.
        clientObject.no = no

        # Member Shear Panel Assigned Members
        clientObject.members = ConvertToDlString(members)

        # Member Shear Panel Assigned Member Sets
        clientObject.member_sets = ConvertToDlString(member_sets)

        # Member Shear Panel User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Member Shear Panel Definition Type
        clientObject.definition_type = 'DEFINITION_TYPE_DEFINE_S_PROV'

        # Member Shear Panel Position On Section
        clientObject.position_on_section = position_on_section.name

        if clientObject.position_on_section == 'POSITION_DEFINE':
            clientObject.position_on_section_value = position_on_section_value

        # Member Shear Panel Stiffness
        clientObject.stiffness = stiffness

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Timber Effective Lengths to client model
        Model.clientModel.service.set_timber_member_shear_panel(clientObject)



from RSTAB.initModel import Model
from RSTAB.enums import ObjectTypes

class GetObjectNumbersByType:

    def __new__(cls,
                ObjectType = ObjectTypes.E_OBJECT_TYPE_NODE,
                model = Model):

        """
        Returns a sorted list which contains object numbers in RSTAB tables.
        ObjectNumberList = [1, 2, ... ]

        Args:
            ObjectType (enum): Object type enum
            model(RSTAB Class, optional): Model to be edited
        Returns:
            Sorted list of object numbers.
        """

        ObjectNumber = Model.clientModel.service.get_all_object_numbers_by_type(ObjectType.name)
        ObjectNumberList = []

        if len(ObjectNumber):
            for i in range(len(ObjectNumber.item)):
                itemCount = 0
                # this is used when requesting objects in loads (E_OBJECT_TYPE_NODAL_LOAD, E_OBJECT_TYPE_LINE_LOAD etc.)
                try:
                    itemCount = ObjectNumber.item[i].children
                # all other objects
                except:
                    itemCount = ObjectNumber.item[i].no
                ObjectNumberList.append(itemCount)

        ObjectNumberList.sort()
        return ObjectNumberList



from RSTAB.initModel import clearAttributes, deleteEmptyAttributes, Model, ConvertStrToListOfInt
from RSTAB.enums import ObjectTypes


class Material():
    def __init__(self,
                 no: int = 1,
                 name: str = 'S235',
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Material Tag
            name (str): Name of Desired Material (As Named in RSTAB Database)
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Material
        clientObject = model.clientModel.factory.create('ns0:material')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Material No.
        clientObject.no = no

        # Material Name
        clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add material to client model
        model.clientModel.service.set_material(clientObject)

    @staticmethod
    def DeleteMaterial(materials_no: str = '1 2', model = Model):

        '''
        Args:
            materials_no (str): Numbers of Materials to be deleted
            model (RSTAB Class, optional): Model to be edited
        '''

        # Delete from client model
        for material in ConvertStrToListOfInt(materials_no):
            model.clientModel.service.delete_object(ObjectTypes.E_OBJECT_TYPE_MATERIAL.name, material)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.dataTypes import inf
from RSTAB.enums import NodalSupportType

def setNodalSupportConditions(clientObject,
                              C_u_X: float,
                              C_u_Y: float,
                              C_u_Z: float,
                              C_phi_X: float,
                              C_phi_Y: float,
                              C_phi_Z: float):
    '''
    Sets Nodal Support Conditions

    Args:
        clientObject: Client Model Object | Nodal Support
        C_u_X,Y,Z (float): Translational Support Conditions in Respected Direction
        C_phi_X,Y,Z (float): Rotational Support Conditions about Respected Axis

    Returns:
        clientObject: Initialized Client Model Object | Nodal Support
    '''

    clientObject.spring_x = C_u_X
    clientObject.spring_y = C_u_Y
    clientObject.spring_z = C_u_Z

    clientObject.rotational_restraint_x = C_phi_X
    clientObject.rotational_restraint_y = C_phi_Y
    clientObject.rotational_restraint_z = C_phi_Z

    return clientObject

class NodalSupport():
    def __init__(self,
                 no: int = 1,
                 nodes_no: str = '1 2',
                 support = NodalSupportType.HINGED,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Nodal Support

        Args:
            no (int): Nodal Support Tag
            nodes_no (str): Assigned to Nodes
            support (enum or list): Support Definition List
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited

        Raises:
            ValueError: 'Support parameter can be enum or list with 6 items.'
        """

        # Client model | Nodal Support
        clientObject = model.clientModel.factory.create('ns0:nodal_support')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Nodal Support No.
        clientObject.no = no

        # Nodes No. (e.g. "5 6 7 12")
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Nodal Support Conditions
        if   support == NodalSupportType.FIXED:
            # FIXED       'xxx xxx'
            clientObject = setNodalSupportConditions(clientObject, inf, inf, inf, inf, inf, inf)

        elif support == NodalSupportType.HINGED:
            # HINGED      'xxx --x'
            clientObject = setNodalSupportConditions(clientObject, inf, inf, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER:
            # ROLLER      '--x --x'
            clientObject = setNodalSupportConditions(clientObject, 0.0, 0.0, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER_IN_X:
            # ROLLER_IN_X '-xx --x'
            clientObject = setNodalSupportConditions(clientObject, 0.0, inf, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER_IN_Y:
            # ROLLER_IN_Y 'x-x --x'
            clientObject = setNodalSupportConditions(clientObject, inf, 0.0, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER_IN_Z:
            # ROLLER_IN_Z 'xx- --x'
            clientObject = setNodalSupportConditions(clientObject, inf, inf, 0.0, 0.0, 0.0, inf)

        elif support == NodalSupportType.FREE:
            # FREE '--- ---'
            clientObject = setNodalSupportConditions(clientObject, 0, 0, 0, 0, 0, 0)

        elif isinstance(support, list) and len(support) == 6:
            clientObject = setNodalSupportConditions(clientObject, support[0], support[1], support[2], support[3], support[4], support[5])

        else:
            raise ValueError('Support parameter can be enum or list with 6 items.')

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Support to client model
        model.clientModel.service.set_nodal_support(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import SteelMemberShearPanelDefinitionType, SteelMemberShearPanelPositionOnSection, SteelMemberShearPanelFasteningArrangement

class SteelMemberShearPanel():
    def __init__(self,
                no: int = 1,
                name: str = '',
                definition_type = SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_TRAPEZOIDAL_SHEETING,
                members: str = "",
                member_sets: str = "",
                categories = [SteelMemberShearPanelPositionOnSection.POSITION_ON_UPPER_FLANGE, "FI (+) 35/207 - 0.63 (b: 1) | DIN 18807 | Fischer Profil", SteelMemberShearPanelFasteningArrangement.FASTENING_ARRANGEMENT_EVERY_RIB],
                parameters = [1, 2, 0.000247, 0.01043],
                comment: str = '',
                params: dict = None):
        """
        Args:
            no (int): Steel Member Shear Panel Tag
            name (str): User Defined Member Shear Panel Name
            definition_type (enum): Steel Member Shear Panel Definition Type Enumeration
            members (str): Assigned Members
            member_sets (str): Assigned Member Sets
            categories (list): Positional Categories LIst
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_TRAPEZOIDAL_SHEETING:
                    categories[0] = Section Position Enumeration Type
                    categories[1] = Sheeting Name
                    categories[2] = Fastening Arrangment Enumeration Type
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_BRACING:
                    categories[0] = Section Position Enumeration Type
                    categories[1] = Diagonal Section Name
                    categories[2] = Post Section Name
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_TRAPEZOIDAL_SHEETING_AND_BRACING:
                    categories[0] = Section Position Enumeration Type
                    categories[1] = Sheeting Name
                    categories[2] = Digonal Section Name
                    categories[3] = Post Section Name
                    categories[4] = Fastening Arrangment Enumeration Type
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_DEFINE_S_PROV:
                    categories[0] = Section Position Enumeration Type
            parameters (list): Positional Parameters List
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_TRAPEZOIDAL_SHEETING:
                    parameters[0] = Panel Length
                    parameters[1] = Beam Spacing
                    parameters[2] = K1 Coefficient
                    parameters[3] = K2 Coefficient
                    if categories[0] == "POSITION_DEFINE":
                        parameters[4] = Position on Section Value
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_BRACING:
                    parameters[0] = Panel Length
                    parameters[1] = Beam Spacing
                    parameters[2] = Post Spacing
                    parameters[3] = Number of Bracings
                    parameters[4] = Diagonals Section Area
                    parameters[5] = Post Section Area
                    if categories[0] == "POSITION_DEFINE":
                        parameters[6] = Position on Section Value
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_TRAPEZOIDAL_SHEETING_AND_BRACING:
                    parameters[0] = Panel Length
                    parameters[1] = Beam Spacing
                    parameters[2] = K1 Coefficient
                    parameters[3] = K2 Coefficient
                    parameters[4] = Post Spacing
                    parameters[5] = Number of Bracing
                    parameters[6] = Diagonals Section Area
                    parameters[7] = Post Section Area
                    if categories[0] == "POSITION_DEFINE":
                        parameters[8] = Position on Section Value
                for definition_type == SteelMemberShearPanelDefinitionType.DEFINITION_TYPE_DEFINE_S_PROV:
                    parameters[0] = Stifness
                    if categories[0] == "POSITION_DEFINE":
                        parameters[1] = Position on Section Value
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
        """

         # Client Model | Types For Steel Design Member Shear Panel
        clientObject = Model.clientModel.factory.create('ns0:steel_member_shear_panel')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Shear Panel No.
        clientObject.no = no

        # Member Shear Panel Assigned Members
        clientObject.members = ConvertToDlString(members)

        # Member Shear Panel Assigned Member Sets
        clientObject.member_sets = ConvertToDlString(member_sets)

        # Member Shear Panel User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Member Shear Panel Definition Type
        clientObject.definition_type = definition_type.name

        # Member Shear Panel Categories
        if definition_type.name == "DEFINITION_TYPE_TRAPEZOIDAL_SHEETING":
            clientObject.position_on_section = categories[0].name
            clientObject.sheeting_name = categories[1]
            clientObject.fastening_arrangement = categories[2].name

        elif definition_type.name == "DEFINITION_TYPE_BRACING":
            clientObject.position_on_section = categories[0].name
            clientObject.diagonals_section_name = categories[1]
            clientObject.posts_section_name = categories[2]

        elif definition_type.name == "DEFINITION_TYPE_TRAPEZOIDAL_SHEETING_AND_BRACING":
            clientObject.position_on_section = categories[0].name
            clientObject.sheeting_name = categories[1]
            clientObject.diagonals_section_name = categories[2]
            clientObject.posts_section_name = categories[3]
            clientObject.fastening_arrangement = categories[4].name

        elif definition_type.name == "DEFINITION_TYPE_DEFINE_S_PROV":
            clientObject.position_on_section = categories[0].name

        # Member Shear Panel Parameters
        if definition_type.name == "DEFINITION_TYPE_TRAPEZOIDAL_SHEETING":
            clientObject.panel_length = parameters[0]
            clientObject.beam_spacing = parameters[1]
            clientObject.coefficient_k1 = parameters[2]
            clientObject.coefficient_k2 = parameters[3]

            if categories[0].name == "POSITION_DEFINE":
                clientObject.position_on_section_value = parameters[4]

        elif definition_type.name == "DEFINITION_TYPE_BRACING":
            clientObject.panel_length = parameters[0]
            clientObject.beam_spacing = parameters[1]
            clientObject.post_spacing = parameters[2]
            clientObject.number_of_bracings = parameters[3]
            clientObject.diagonals_section_area = parameters[4]
            clientObject.posts_section_area = parameters[5]

            if categories[0].name == "POSITION_DEFINE":
                clientObject.position_on_section_value = parameters[6]

        elif definition_type.name == "DEFINITION_TYPE_TRAPEZOIDAL_SHEETING_AND_BRACING":
            clientObject.panel_length = parameters[0]
            clientObject.beam_spacing = parameters[1]
            clientObject.coefficient_k1 = parameters[2]
            clientObject.coefficient_k2 = parameters[3]
            clientObject.post_spacing = parameters[4]
            clientObject.number_of_bracings = parameters[5]
            clientObject.diagonals_section_area = parameters[6]
            clientObject.posts_section_area = parameters[7]

            if categories[0].name == "POSITION_DEFINE":
                clientObject.position_on_section_value = parameters[8]

        elif definition_type.name == "DEFINITION_TYPE_DEFINE_S_PROV":
            clientObject.stiffness = parameters[0]

            if categories[0].name == "POSITION_DEFINE":
                clientObject.position_on_section_value = parameters[1]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Steel Effective Lengths to client model
        Model.clientModel.service.set_steel_member_shear_panel(clientObject)
